/***********************************************************************/
/*                                                                     */
/*  PROJECT		  :FTF Design Challenge Europe 2008   	                 */
/*  AUTHOR      :Carlos Cossio                                         */
/*  FILE        :RTC.c                                                 */
/*  DATE        :June 21, 2008                                         */
/*  DESCRIPTION :This file allows to manage the Real Time Clock (RTC)  */
/*                                                                     */
/***********************************************************************/


#include "RTC.h"


/******************************************************************************
Name       : RTC_Init
Author     : Carlos Cossio
Parameters : None
Returns    : None
Description: Setup the RTC driver
******************************************************************************/
void RTC_Init (void)
{
  RTCSC_RTIF = 1;                         // Clear RTIF flag
  RTCSC_RTIE = 0;                         // Disable interrupt
  RTCSC_RTCLKS = 0;                       // RTC clock source: 1 kHz low-power oscillator
}


/******************************************************************************
Name       : RTC_Start
Author     : Carlos Cossio
Parameters : prescaler -> RTC prescaler divide-by value 
Returns    : None
Description: Start the RTC counter
******************************************************************************/
void RTC_Start (unsigned char prescaler)
{
  RTCSC_RTCPS = prescaler;                // Set prescaler period
}


/******************************************************************************
Name       : RTC_Stop
Author     : Carlos Cossio
Parameters : None
Returns    : None
Description: Stop the RTC counter
******************************************************************************/
void RTC_Stop (void)
{
  RTCSC_RTIF = 1;                         // Clear RTIF flag
  RTCSC_RTCPS = RTC_OFF;                  // Prescaler off
}


/******************************************************************************
Name       : RTC_Delay
Author     : Carlos Cossio
Parameters : milliseconds -> Number of milliseconds
Returns    : None
Description: Wait for a number of milliseconds
******************************************************************************/
void RTC_Delay (unsigned int milliseconds)
{	
  RTCSC_RTIF = 1;                         // Clear RTIF flag

  if (milliseconds >= 1000) {
    RTCMOD = (char)(milliseconds / 1000); // To count in seconds
    RTC_Start (DIVIDE_1000);              // Time base: 1 second
  } 
  else if (milliseconds >= 100) {
    RTCMOD = (char)(milliseconds / 100);  // To count in hundreds of milliseconds
    RTC_Start (DIVIDE_100);               // Time base: 100 ms
  }
  else if (milliseconds >= 10) {
    RTCMOD = (char)(milliseconds / 10);   // To count in tens of milliseconds
    RTC_Start (DIVIDE_10);                // Time base: 10 ms
  } 
  else {
    RTCMOD = (char)milliseconds;          // To count in units of milliseconds
    RTC_Start (DIVIDE_1);                 // Time base: 1 ms
  }
  
  while (!RTCSC_RTIF);                    // Wait the requested time
  
  RTC_Stop();                             // Stop the RTC
}


/******************************************************************************
Name       : RTC_Enable
Author     : Carlos Cossio
Parameters : None
Returns    : None
Description: Enable RTC interrupts every half a second
******************************************************************************/
void RTC_Enable (void)
{
  RTCSC_RTIF = 1;                         // Clear RTIF flag
  RTCSC_RTIE = 1;                         // Enable interrupt
  
  RTCMOD = 5;                             // Interrupt every half a second
  RTC_Start (DIVIDE_100);                 // Time base: 100 milliseconds
}


/******************************************************************************
Name       : RTC_Disable
Author     : Carlos Cossio
Parameters : None 
Returns    : None
Description: Disable RTC interrupts
******************************************************************************/
void RTC_Disable (void)
{
  RTCSC_RTIF = 1;                         // Clear RTIF flag
  RTCSC_RTIE = 0;                         // Disable interrupt
}


/******************************************************************************
Name       : RTC_ISR
Author     : Carlos Cossio
Parameters : None
Returns    : None
Description: RTC interrupt service routine every half a second
******************************************************************************/
interrupt VectorNumber_Vrtc void RTC_ISR (void)
{
  RTC_Stop();                             // Stop RTC counter
  RTC_Disable();                          // Disable interrupt
}
